

pact_cmd_schema = {
  "$schema": "http://json-schema.org/draft-04/schema#",
  "type": "object",
  "properties": {
    "hash": {
      "type": "string"
    },
    "sigs": {
      "type": "array",
      "items": [
        {
          "type": "object",
          "properties": {
            "sig": {
              "type": "string"
            }
          },
          "required": [
            "sig"
          ]
        }
      ]
    },
    "cmd": {
      "type": "string"
    }
  },
  "required": [
    "hash",
    "sigs",
    "cmd"
  ]
}

cmd_schema = {
  "$schema": "http://json-schema.org/draft-04/schema#",
  "type": "object",
  "properties": {
    "networkId": {
      "type": "string"
    },
    "payload": {
      "type": "object",
      "properties": {
        "exec": {
          "type": "object",
          "properties": {
            "data": {
              "type": "object",
              "properties": {
                "device_exec": {
                  "type": "object"
                }
              },
              "required": [
                "device_exec"
              ]
            },
            "code": {
              "type": "string"
            }
          },
          "required": [
            "data",
            "code"
          ]
        }
      },
      "required": [
        "exec"
      ]
    },
    "signers": {
      "type": "array",
      "items": {}
    },
    "meta": {
      "type": "object",
      "properties": {
        "sender": {
          "type": "string"
        },
        "creationTime": {
          "type": "integer"
        },
        "gasLimit": {
          "type": "integer"
        },
        "gasPrice": {
          "type": "number"
        },
        "chainId": {
          "type": "string"
        },
        "ttl": {
          "type": "integer"
        }
      },
      "required": [
        "sender",
        "creationTime",
        "gasLimit",
        "gasPrice",
        "chainId",
        "ttl"
      ]
    },
    "nonce": {
      "type": "string"
    }
  },
  "required": [
    "networkId",
    "payload",
    "signers",
    "meta",
    "nonce"
  ]
}



from pypact.utils import get_headers
from data_shipper import utils
import requests as rt
from jsonschema import validate
from data_shipper import schema
import json


def check_auth(cmd, network_id):
    res = rt.post(utils.get_api_host(network_id)+'/api/v1/local', json=cmd, headers=get_headers())
    if res.ok:
        if res.json()['result']['status'] == 'success':
            return True
        else:
            return False
    else:
        return False


def validate_device_id(device_id, msg):
    try:
        validate(instance=msg, schema=schema.pact_cmd_schema)
        cmd = json.loads(msg['cmd'])
        validate(instance=cmd, schema=schema.cmd_schema)
        matched, d_id = utils.extract_device_id(cmd['payload']['exec']['code'])
        if matched:
            if d_id == device_id:
                return True
            else:

                return False
        else:
            return False
    except Exception as e:
        print(e.__str__())
        return False



import hashlib
import json
import time
import re
from data_shipper import config
from pypact.pact import Pact


def get_data_hash(data):
    return hashlib.sha256(json.dumps(data).encode('utf-8')).hexdigest()


def default_meta(sender="not real"):
    pact = Pact()
    return pact.lang.mk_meta(sender, config.chain_id, 0.0000001, 3000, time.time().__round__()-15, 28800)


def extract_device_id(pact_code):
    regex = r"(?<=\(free\.cyberfly_devices\.auth-device\s\")([0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12})"
    match = re.search(regex, pact_code)
    if match:
        return True, match.group(1)
    else:
        return False, "Invalid code"


def get_api_host(network_id):
    if network_id == "testnet04":
        return "https://api.testnet.chainweb.com/chainweb/0.0/testnet04/chain/{}/pact".format(config.chain_id)
    else:
        return "https://api.chainweb.com/chainweb/0.0/mainnet01/chain/{}/pact".format(config.chain_id)



from setuptools import setup, find_packages
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()
setup(
    name='cyberfly-data-shipper',
    version='0.1.1',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'requests',
        'paho-mqtt',
        'pypact-lang'
    ],
    url='https://github.com/cyberfly-io/data-shipper',
    long_description=long_description,
    long_description_content_type='text/markdown'
)


import json

from typing import Callable
import paho.mqtt.client as mqtt

from data_shipper import config, auth

mqttc = mqtt.Client(clean_session=True)


class CyberflyDataShipper:
    def __init__(self, device_id: str, key_pair: dict, network_id: str = "mainnet01"):
        self.key_pair = key_pair
        self.network_id = network_id
        self.device_data = {}
        self.device_id = device_id
        self.account = "k:" + self.key_pair.get("publicKey")
        self.caller = default_caller
        self.mqtt_client = mqttc
        self.topic = device_id
        self.mqtt_client.user_data_set(self)
        self.mqtt_client.on_connect = on_connect
        self.mqtt_client.on_message = on_received
        self.run(config.mqtt_broker, config.mqtt_port)

    def update_data(self, key: str, value):
        self.device_data.update({key: value})

    def on_message(self) -> Callable:
        def decorator(callback_function):
            self.caller = callback_function

        return decorator

    def run(self, host: str, port: int) -> None:
        try:
            self.mqtt_client.connect(
            host, port, 60)
        except Exception as e:
            print(e.__str__())
        self.mqtt_client.loop_start()


def on_connect(client: mqtt.Client, mqtt_class: CyberflyDataShipper, __flags, received_code: int) -> None:
    print("Connected with result code " + str(received_code))
    client.subscribe(mqtt_class.topic)


def on_received(__client: mqtt.Client, mqtt_class: CyberflyDataShipper, msg: mqtt.MQTTMessage) -> None:
    json_string = msg.payload.decode("utf-8")
    try:
        json_data = json.loads(json_string)
        if auth.validate_device_id(mqtt_class.device_id, json_data) and auth.check_auth(json_data, mqtt_class.network_id):
            try:
                mqtt_class.caller(json.loads(json_data['cmd'])['payload']['exec']['data']['device_exec'])
            except Exception as e:
                print(e.__str__())
        else:
            print("auth failed")
    except Exception as e:
        print(e.__str__())


def default_caller(data):
    pass
